/*
* Original code by ©Kosada Incorporated, released under MIT License, modified by Bodysoulspirit.
* This code may be modified and distributed under the terms of the MIT License.
* For more information, see https://vuo.org/license.
*/

#include "node.h"

VuoModuleMetadata({
					 "title" : "Build List with Data",
					 "keywords" : [ "iterate", "repeat", "multiple", "many", "foreach", "each", "loop", "while", "cycle" ],
					 "description" : "Same as the stock node version, but allows to retrieve the amount of items to be built + feed other data in the loop without the need of extra nodes.",
					 "version" : "1.0.0",
				 });

struct nodeInstanceData
{
	VuoInteger count;
	VuoInteger finalCount;
	VuoList_VuoGenericType1 list;
	VuoGenericType2 ownData;
	VuoInteger savedAmount;
};

struct nodeInstanceData *nodeInstanceInit(void)
{
	struct nodeInstanceData *context = (struct nodeInstanceData *)calloc(1,sizeof(struct nodeInstanceData));
	VuoRegister(context, free);
	return context;
}

void nodeInstanceTriggerStart
(
		VuoInstanceData(struct nodeInstanceData *) context
)
{
	if ((*context)->list)
	{
		VuoRelease((*context)->list);
		(*context)->list = NULL;
	}
}

void nodeInstanceEvent
(
		VuoInputData(VuoInteger, {"name":"Amount", "default":10, "suggestedMin":1}) fire,
		VuoInputEvent({"data":"fire", "eventBlocking":"none"}) fireEvent,
		VuoInputData(VuoGenericType2) feedData,
		VuoInputEvent({"data":"feedData", "eventBlocking":"wall"}) feedDataEvent,
		VuoInputData(VuoGenericType1) builtItem,
		VuoInputEvent({"data":"builtItem", "eventBlocking":"none"}) builtItemEvent,
		VuoOutputTrigger(builtList, VuoList_VuoGenericType1, {"name":"Finished Building"}),
		VuoOutputTrigger(buildItem, VuoInteger, {"name":"Index"}),
		VuoOutputTrigger(amountOut, VuoInteger, {"name":"Amount Out"}),
		VuoOutputTrigger(fedData, VuoGenericType2, {"name":"Fed Data"}),
		VuoInstanceData(struct nodeInstanceData *) context
)
{
	// Only allow creating a new list if we've received back _all_ the fired events,
	// to prevent outputting partially-complete lists
	// when `fire` receives frequent events but the feedback loop is slow.
	if (fireEvent && !(*context)->list)
	{
		if (fire < 1)
		{
			builtList(VuoListCreate_VuoGenericType1());
			return;
		}

		(*context)->list = VuoListCreate_VuoGenericType1();
		VuoRetain((*context)->list);

		(*context)->ownData = feedData;
		fedData((*context)->ownData);

		(*context)->savedAmount = fire;
		amountOut((*context)->savedAmount);

		(*context)->count = 1;
		buildItem((*context)->count);

		(*context)->finalCount = fire;
	}

	if (builtItemEvent && (*context)->list && (*context)->count <= (*context)->finalCount)
	{
		VuoListAppendValue_VuoGenericType1((*context)->list, builtItem);

		++(*context)->count;
		if ((*context)->count <= (*context)->finalCount)
			buildItem((*context)->count);
		else
		{
			builtList((*context)->list);

			VuoRelease((*context)->list);
			(*context)->count = 0;
			(*context)->list = NULL;
		}
	}
}

void nodeInstanceFini(
		VuoInstanceData(struct nodeInstanceData *) context
)
{
	if ((*context)->list)
		VuoRelease((*context)->list);
}
